/**
 * PauWare engine software (http://www.PauWare.com) InterDeposit Digital Number
 * IDDN.FR.001.360023.000.S.P.2006.000.10000. Use of this software is subject to
 * the restrictions of the LGPL license version 3
 * http://www.gnu.org/licenses/lgpl-3.0.en.html
 */
package com.pauware.pauware_engine._Core;

/**
 * This class represents the general notion of <I>Transition</I> in UML.
 * <p>
 * Compatibility: <I>PauWare Java EE</I> (Java SE/Java EE) and <I>PauWare Java
 * ME</I> (Java SE/Java ME).
 *
 * @author Franck.Barbier@FranckBarbier.com
 * @version 1.3 (February 2015)
 * @since 1.0
 */
public class Transition {

    /**
     * This state at the source of the transition.
     */
    protected final AbstractStatechart _from;
    /**
     * This state at the end of the transition.
     */
    protected final AbstractStatechart _to;

    /**
     * This constructor creates a transition instance requiring that
     * <CODE>from</CODE> is not equal to <CODE>null</CODE> <b>and</b>
     * <CODE>to</CODE> is not equal to <CODE>null</CODE>. This pre-condition is
     * checked within the
     * {@link AbstractStatechart_monitor#fires(String,AbstractStatechart,AbstractStatechart,Object,String,Object[],Object,String,Object[],byte)}
     * method.
     */
    protected Transition(AbstractStatechart from, AbstractStatechart to) {
        _from = from;
        _to = to;
    }

    /**
     * This method is used when a <I>Transition</I> is put in a hashtable as a
     * key.
     */
    public boolean equals(Object transition) {
        if (this == transition) {
            return true;
        }
        if (transition instanceof Transition) {
            return _from.equals(((Transition) transition)._from) && _to.equals(((Transition) transition)._to);
        }
        return false;
    }

    /**
     * This method is used when a <I>Transition</I> is put in a hashtable as a
     * key.
     */
    public int hashCode() {
        return _from.hashCode() * 31 + _to.hashCode();
    }

    /**
     * This method is an accessor for the {@link Transition#_from} field.
     */
    public AbstractStatechart from() {
        return _from;
    }

    /**
     * This method is an accessor for the {@link Transition#_to} field.
     */
    public AbstractStatechart to() {
        return _to;
    }
}
